/* 
 * Mach Operating System
 * Copyright (c) 1993,1992,1991,1990,1989,1988,1987 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator   or   Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they made and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
/*
 * HISTORY
 * $Log:	ipc_swap.h,v $
 * Revision 2.2  93/08/10  13:13:30  mrt
 * 	First checkin to Mach 3.0 RCS tree
 * 
 * Revision 2.1  90/10/27  20:44:22  dpj
 * Moving under MACH3 source control
 * 
 * Revision 1.6  89/05/02  11:10:26  dpj
 * 	Fixed all files to conform to standard copyright/log format
 * 
 *  2-Jan-87  Robert Sansom (rds) at Carnegie Mellon University
 *	Added external definitions of swap_short_sbuf and swap_long_sbuf.
 *
 * 15-Nov-86  Daniel Julin (dpj) at Carnegie-Mellon University
 *	Created.
 *
 *
 */
/*
 * ipc_swap.h 
 *
 *
 * $Header: ipc_swap.h,v 2.2 93/08/10 13:13:30 mrt Exp $ 
 *
 */

/*
 * Definitions and macros for byte-swapping operations in the IPC module. 
 */


#ifndef	_IPC_SWAP_
#define	_IPC_SWAP_

#include	"config.h"

/*
 * TypeType seen in reverse for byte-swapping. 
 */
#if	VaxOrder
typedef struct {
	unsigned int
		msg_type_name:8,
		msg_type_size:8,
		msg_type_numhigh:8,
		msg_type_dummy:1,
		msg_type_deallocate:1,
		msg_type_longform:1,
		msg_type_inline:1,
		msg_type_numlow:4;
} swap_msg_type_t;
#else	VaxOrder
typedef struct {
	unsigned int
		msg_type_name:8,
		msg_type_size:8,
		msg_type_numlow:8,
		msg_type_dummy:1,
		msg_type_deallocate:1,
		msg_type_longform:1,
		msg_type_inline:1,
		msg_type_numhigh:4;
} swap_msg_type_t;
#endif	VaxOrder

/*
 * Declarations needed for the following macros to work. 
 */
#define	SWAP_DECLS	register unsigned long swap_tmp

/*
 * Swap and copy a long value, and return the swapped value. 
 */
#define	SWAP_LONG(f,t) (						\
	swap_tmp = f,							\
	t = ((swap_tmp >> 24) | ((swap_tmp >> 8) & 0xff00)		\
		| ((swap_tmp << 8) & 0xff0000) | (swap_tmp << 24))	\
)

/*
 * Swap and copy a short value, and return the swapped value. 
 */
#define	SWAP_SHORT(f,t) (						\
	swap_tmp = f,							\
	t = ((swap_tmp >> 8) | ((swap_tmp << 8) & 0xff00))		\
)

/*
 * Swap in place an array of longs. 
 */
#define	SWAP_LONG_ARRAY(s,n) {						\
	register long	*p = (long *)s;					\
									\
	for (; n; n--, p++) SWAP_LONG(*p,*p);				\
}

/*
 * Swap in place an array of shorts. 
 */
#define	SWAP_SHORT_ARRAY(s,n) {						\
	register short	*p = (short *)s;				\
									\
	for (; n; n--, p++) SWAP_SHORT(*p,*p);				\
}

/*
 * External definitions for functions implemented by ipc_swap.c
 */

extern void swap_long_sbuf();
/*
sbuf_ptr_t	sb_ptr;
sbuf_pos_t	*pos_ptr;
long		**to_ptr;
int		count;
*/

extern void swap_short_sbuf();
/*
sbuf_ptr_t	sb_ptr;
sbuf_pos_t	*pos_ptr;
short		**to_ptr;
int		count;
*/

#endif	_IPC_SWAP_
