/* 
 * Mach Operating System
 * Copyright (c) 1993,1992,1991,1990,1989,1988,1987 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator   or   Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they made and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
/*
 * HISTORY
 * $Log:	key_defs.h,v $
 * Revision 2.2  93/08/10  13:13:38  mrt
 * 	First checkin to Mach 3.0 RCS tree
 * 
 * Revision 2.1  90/10/27  20:44:23  dpj
 * Moving under MACH3 source control
 * 
 * Revision 1.7  89/05/02  11:10:31  dpj
 * 	Fixed all files to conform to standard copyright/log format
 * 
 *  5-Jun-87  Robert Sansom (rds) at Carnegie Mellon University
 *	Added macros to convert keys between network and host order.
 *
 * 12-Apr-87  Robert Sansom (rds) at Carnegie Mellon University
 *	Added KEY_IS_NULL.
 *
 *  2-Feb-87  Robert Sansom (rds) at Carnegie Mellon University
 *	Added KEY_EQUAL.
 *
 *  5-Nov-86  Robert Sansom (rds) at Carnegie-Mellon University
 *	Started.
 *
 */
/*
 * key_defs.h
 *
 *
 * $Header: key_defs.h,v 2.2 93/08/10 13:13:38 mrt Exp $
 *
 */

/*
 * Definitions of encryption keys etc..
 */


#ifndef	_KEY_DEFS_
#define	_KEY_DEFS_

/*
 * An encrytion key.
 */
typedef union {
    unsigned char	key_bytes[16];
    unsigned long	key_longs[4];
} key_t, *key_ptr_t;

#define KEY_EQUAL(key1, key2) 					\
    ((key1.key_longs[0] == key2.key_longs[0])			\
	&& (key1.key_longs[1] == key2.key_longs[1])		\
	&& (key1.key_longs[2] == key2.key_longs[2])		\
	&& (key1.key_longs[3] == key2.key_longs[3]))

#define KEY_IS_NULL(key)					\
    (((key).key_longs[0] == 0) && ((key).key_longs[1] == 0)	\
	&& ((key).key_longs[2] == 0) && ((key).key_longs[3] == 0))


/*
 * Macros to convert keys between network and host byte order.
 */
#define NTOH_KEY(key) {							\
    (key).key_longs[0] = ntohl((key).key_longs[0]);			\
    (key).key_longs[1] = ntohl((key).key_longs[1]);			\
    (key).key_longs[2] = ntohl((key).key_longs[2]);			\
    (key).key_longs[3] = ntohl((key).key_longs[3]);			\
}

#define HTON_KEY(key) {							\
    (key).key_longs[0] = htonl((key).key_longs[0]);			\
    (key).key_longs[1] = htonl((key).key_longs[1]);			\
    (key).key_longs[2] = htonl((key).key_longs[2]);			\
    (key).key_longs[3] = htonl((key).key_longs[3]);			\
}

/*
 * Structure used to transmit or store a token or a key.
 */
typedef union {
    key_t	si_key;
    key_t	si_token;
} secure_info_t, *secure_info_ptr_t;

/*
 * Security Level of ports and messages.
 */
#define PORT_NOT_SECURE		0
#define MESSAGE_NOT_SECURE	0

#endif	_KEY_DEFS_
