/* 
 * Mach Operating System
 * Copyright (c) 1993 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator   or   Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they made and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
/*
 * HISTORY
 * $Log:	logstatServer.c,v $
 * Revision 2.3  93/08/11  14:31:03  mrt
 * 	dumped an dangling #else clause.
 * 	[93/08/10            mrt]
 * 
 * Revision 2.2  93/08/10  13:16:57  mrt
 * 	First checkin to Mach 3.0 RCS tree
 * 
 */

/* Module logstat */

#define EXPORT_BOOLEAN
#include <mach/boolean.h>
#include <mach/message.h>
#include <mach/mig_errors.h>
#include <mach/mig_support.h>

#ifndef	mig_internal
#define	mig_internal	static
#endif

#ifndef	TypeCheck
#define	TypeCheck 1
#endif

#ifndef	UseExternRCSId
#define	UseExternRCSId		1
#endif

#define msg_request_port	msg_local_port
#define msg_reply_port		msg_remote_port
#include "ls_defs.h"

/* Routine ls_sendlog */
mig_internal void _Xls_sendlog
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
		msg_type_long_t old_log_ptrType;
		log_ptr_t old_log_ptr;
		msg_type_long_t cur_log_ptrType;
		log_ptr_t cur_log_ptr;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_sendlog
		(port_t server_port, log_ptr_t *old_log_ptr, unsigned int *old_log_ptrCnt, log_ptr_t *cur_log_ptr, unsigned int *cur_log_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

	static msg_type_long_t old_log_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_UNSTRUCTURED,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};
	static msg_type_long_t cur_log_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_UNSTRUCTURED,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};

	unsigned int old_log_ptrCnt;
	unsigned int cur_log_ptrCnt;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_sendlog(In0P->Head.msg_request_port, &OutP->old_log_ptr, &old_log_ptrCnt, &OutP->cur_log_ptr, &cur_log_ptrCnt);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 64;

	OutP->old_log_ptrType = old_log_ptrType;
	OutP->old_log_ptrType.msg_type_long_number /* old_log_ptrCnt */ = /* old_log_ptrType.msg_type_long_number */ old_log_ptrCnt;

	OutP->cur_log_ptrType = cur_log_ptrType;
	OutP->cur_log_ptrType.msg_type_long_number /* cur_log_ptrCnt */ = /* cur_log_ptrType.msg_type_long_number */ cur_log_ptrCnt;

	OutP->Head.msg_simple = FALSE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_resetlog */
mig_internal void _Xls_resetlog
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_resetlog(port_t server_port);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_resetlog(In0P->Head.msg_request_port);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 32;

	OutP->Head.msg_simple = TRUE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_writelog */
mig_internal void _Xls_writelog
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_writelog(port_t server_port);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_writelog(In0P->Head.msg_request_port);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 32;

	OutP->Head.msg_simple = TRUE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_sendstat */
mig_internal void _Xls_sendstat
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
		msg_type_long_t stat_ptrType;
		stat_ptr_t stat_ptr;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_sendstat
		(port_t server_port, stat_ptr_t *stat_ptr, unsigned int *stat_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

	static msg_type_long_t stat_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_INTEGER_32,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};

	unsigned int stat_ptrCnt;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_sendstat(In0P->Head.msg_request_port, &OutP->stat_ptr, &stat_ptrCnt);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 48;

	OutP->stat_ptrType = stat_ptrType;
	OutP->stat_ptrType.msg_type_long_number /* stat_ptrCnt */ = /* stat_ptrType.msg_type_long_number */ stat_ptrCnt;

	OutP->Head.msg_simple = FALSE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_resetstat */
mig_internal void _Xls_resetstat
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_resetstat(port_t server_port);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_resetstat(In0P->Head.msg_request_port);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 32;

	OutP->Head.msg_simple = TRUE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_senddebug */
mig_internal void _Xls_senddebug
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
		msg_type_long_t debug_ptrType;
		debug_ptr_t debug_ptr;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_senddebug
		(port_t server_port, debug_ptr_t *debug_ptr, unsigned int *debug_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

	static msg_type_long_t debug_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_INTEGER_32,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};

	unsigned int debug_ptrCnt;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_senddebug(In0P->Head.msg_request_port, &OutP->debug_ptr, &debug_ptrCnt);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 48;

	OutP->debug_ptrType = debug_ptrType;
	OutP->debug_ptrType.msg_type_long_number /* debug_ptrCnt */ = /* debug_ptrType.msg_type_long_number */ debug_ptrCnt;

	OutP->Head.msg_simple = FALSE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_setdebug */
mig_internal void _Xls_setdebug
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
		msg_type_long_t debug_ptrType;
		debug_ptr_t debug_ptr;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_setdebug
		(port_t server_port, debug_ptr_t debug_ptr, unsigned int debug_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 40) || (msg_simple != FALSE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

#if	TypeCheck
	if ((In0P->debug_ptrType.msg_type_header.msg_type_inline != FALSE) ||
	    (In0P->debug_ptrType.msg_type_header.msg_type_longform != TRUE) ||
	    (In0P->debug_ptrType.msg_type_long_name != MSG_TYPE_INTEGER_32) ||
	    (In0P->debug_ptrType.msg_type_long_size != 32))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; goto punt0; }
#define	label_punt0
#endif	TypeCheck

	OutP->RetCode = ls_setdebug(In0P->Head.msg_request_port, In0P->debug_ptr, In0P->debug_ptrType.msg_type_long_number);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 32;

	OutP->Head.msg_simple = TRUE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_sendparam */
mig_internal void _Xls_sendparam
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
		msg_type_long_t param_ptrType;
		param_ptr_t param_ptr;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_sendparam
		(port_t server_port, param_ptr_t *param_ptr, unsigned int *param_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

	static msg_type_long_t param_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_INTEGER_32,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};

	unsigned int param_ptrCnt;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_sendparam(In0P->Head.msg_request_port, &OutP->param_ptr, &param_ptrCnt);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 48;

	OutP->param_ptrType = param_ptrType;
	OutP->param_ptrType.msg_type_long_number /* param_ptrCnt */ = /* param_ptrType.msg_type_long_number */ param_ptrCnt;

	OutP->Head.msg_simple = FALSE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_setparam */
mig_internal void _Xls_setparam
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
		msg_type_long_t param_ptrType;
		param_ptr_t param_ptr;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_setparam
		(port_t server_port, param_ptr_t param_ptr, unsigned int param_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 40) || (msg_simple != FALSE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

#if	TypeCheck
	if ((In0P->param_ptrType.msg_type_header.msg_type_inline != FALSE) ||
	    (In0P->param_ptrType.msg_type_header.msg_type_longform != TRUE) ||
	    (In0P->param_ptrType.msg_type_long_name != MSG_TYPE_INTEGER_32) ||
	    (In0P->param_ptrType.msg_type_long_size != 32))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; goto punt0; }
#define	label_punt0
#endif	TypeCheck

	OutP->RetCode = ls_setparam(In0P->Head.msg_request_port, In0P->param_ptr, In0P->param_ptrType.msg_type_long_number);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 32;

	OutP->Head.msg_simple = TRUE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_sendportstat */
mig_internal void _Xls_sendportstat
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
		msg_type_long_t port_stat_ptrType;
		port_stat_ptr_t port_stat_ptr;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_sendportstat
		(port_t server_port, port_stat_ptr_t *port_stat_ptr, unsigned int *port_stat_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

	static msg_type_long_t port_stat_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_UNSTRUCTURED,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};

	unsigned int port_stat_ptrCnt;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_sendportstat(In0P->Head.msg_request_port, &OutP->port_stat_ptr, &port_stat_ptrCnt);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 48;

	OutP->port_stat_ptrType = port_stat_ptrType;
	OutP->port_stat_ptrType.msg_type_long_number /* port_stat_ptrCnt */ = /* port_stat_ptrType.msg_type_long_number */ port_stat_ptrCnt;

	OutP->Head.msg_simple = FALSE;
	OutP->Head.msg_size = msg_size;
}

/* Routine ls_mem_list */
mig_internal void _Xls_mem_list
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	typedef struct {
		msg_header_t Head;
	} Request;

	typedef struct {
		msg_header_t Head;
		msg_type_t RetCodeType;
		kern_return_t RetCode;
		msg_type_long_t class_ptrType;
		mem_class_ptr_t class_ptr;
		msg_type_long_t nam_ptrType;
		mem_nam_ptr_t nam_ptr;
		msg_type_long_t bucket_ptrType;
		mem_bucket_ptr_t bucket_ptr;
	} Reply;

	register Request *In0P = (Request *) InHeadP;
	register Reply *OutP = (Reply *) OutHeadP;
	extern kern_return_t ls_mem_list
		(port_t server_port, mem_class_ptr_t *class_ptr, unsigned int *class_ptrCnt, mem_nam_ptr_t *nam_ptr, unsigned int *nam_ptrCnt, mem_bucket_ptr_t *bucket_ptr, unsigned int *bucket_ptrCnt);

#if	TypeCheck
	boolean_t msg_simple;
#endif	TypeCheck

	unsigned int msg_size;

	static msg_type_long_t class_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_INTEGER_32,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};
	static msg_type_long_t nam_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_CHAR,
		/* msg_type_long_size = */	8,
		/* msg_type_long_number = */	0
	};
	static msg_type_long_t bucket_ptrType = {
	{
		/* msg_type_name = */		0,
		/* msg_type_size = */		0,
		/* msg_type_number = */		0,
		/* msg_type_inline = */		FALSE,
		/* msg_type_longform = */	TRUE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	},
		/* msg_type_long_name = */	MSG_TYPE_INTEGER_32,
		/* msg_type_long_size = */	32,
		/* msg_type_long_number = */	0
	};

	unsigned int class_ptrCnt;
	unsigned int nam_ptrCnt;
	unsigned int bucket_ptrCnt;

#if	TypeCheck
	msg_size = In0P->Head.msg_size;
	msg_simple = In0P->Head.msg_simple;
	if ((msg_size != 24) || (msg_simple != TRUE))
		{ OutP->RetCode = MIG_BAD_ARGUMENTS; return; }
#endif	TypeCheck

	OutP->RetCode = ls_mem_list(In0P->Head.msg_request_port, &OutP->class_ptr, &class_ptrCnt, &OutP->nam_ptr, &nam_ptrCnt, &OutP->bucket_ptr, &bucket_ptrCnt);
#ifdef	label_punt0
#undef	label_punt0
punt0:
#endif	label_punt0
	if (OutP->RetCode != KERN_SUCCESS)
		return;

	msg_size = 80;

	OutP->class_ptrType = class_ptrType;
	OutP->class_ptrType.msg_type_long_number /* class_ptrCnt */ = /* class_ptrType.msg_type_long_number */ class_ptrCnt;

	OutP->nam_ptrType = nam_ptrType;
	OutP->nam_ptrType.msg_type_long_number /* nam_ptrCnt */ = /* nam_ptrType.msg_type_long_number */ nam_ptrCnt;

	OutP->bucket_ptrType = bucket_ptrType;
	OutP->bucket_ptrType.msg_type_long_number /* bucket_ptrCnt */ = /* bucket_ptrType.msg_type_long_number */ bucket_ptrCnt;

	OutP->Head.msg_simple = FALSE;
	OutP->Head.msg_size = msg_size;
}

boolean_t logstat_server
	(msg_header_t *InHeadP, msg_header_t *OutHeadP)
{
	register msg_header_t *InP =  InHeadP;
	register death_pill_t *OutP = (death_pill_t *) OutHeadP;

	static msg_type_t RetCodeType = {
		/* msg_type_name = */		MSG_TYPE_INTEGER_32,
		/* msg_type_size = */		32,
		/* msg_type_number = */		1,
		/* msg_type_inline = */		TRUE,
		/* msg_type_longform = */	FALSE,
		/* msg_type_deallocate = */	FALSE,
		/* msg_type_unused = */		0
	};

	OutP->Head.msg_simple = TRUE;
	OutP->Head.msg_size = sizeof *OutP;
	OutP->Head.msg_type = InP->msg_type;
	OutP->Head.msg_local_port = PORT_NULL;
	OutP->Head.msg_remote_port = InP->msg_reply_port;
	OutP->Head.msg_id = InP->msg_id + 100;

	OutP->RetCodeType = RetCodeType;
	OutP->RetCode = MIG_BAD_ID;

	if ((InP->msg_id > 1210) || (InP->msg_id < 1200))
		return FALSE;
	else {
		typedef void (*SERVER_STUB_PROC)
			(msg_header_t *, msg_header_t *);
		static SERVER_STUB_PROC routines[] = {
			_Xls_sendlog,
			_Xls_resetlog,
			_Xls_writelog,
			_Xls_sendstat,
			_Xls_resetstat,
			_Xls_senddebug,
			_Xls_setdebug,
			_Xls_sendparam,
			_Xls_setparam,
			_Xls_sendportstat,
			_Xls_mem_list,
		};

		if (routines[InP->msg_id - 1200])
			(routines[InP->msg_id - 1200]) (InP, &OutP->Head);
		 else
			return FALSE;
	}
	return TRUE;
}
