/* 
 * Mach Operating System
 * Copyright (c) 1993,1992,1991,1990,1989,1988,1987 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator   or   Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they made and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
/*
 * HISTORY
 * $Log:	nn_defs.h,v $
 * Revision 2.2  93/08/10  13:21:10  mrt
 * 	First checkin to Mach 3.0 RCS tree
 * 
 * Revision 2.1  90/10/27  20:45:30  dpj
 * Moving under MACH3 source control
 * 
 * Revision 1.6  89/05/02  11:14:43  dpj
 * 	Fixed all files to conform to standard copyright/log format
 * 
 * 26-Mar-88  Daniel Julin (dpj) at Carnegie-Mellon University
 *	Converted to use the new memory management module.
 *
 * 27-Apr-87  Robert Sansom (rds) at Carnegie Mellon University
 *	Placed lock and condition inline in request record.
 *	Changed nn_table to consist of inline lock_queue records.
 *
 * 23-Dec-86  Robert Sansom (rds) at Carnegie Mellon University
 *	Started.
 *
 */
/*
 * nn_defs.h
 *
 *
 * $Header: nn_defs.h,v 2.2 93/08/10 13:21:10 mrt Exp $
 *
 */

/*
 * Internal definitions for the network name service module.
 */


#ifndef	_NN_DEFS_
#define	_NN_DEFS_

#include <ctype.h>

#include "mem.h"
#include "disp_hdr.h"
#include "lock_queue.h"
#include "netname_defs.h"
#include "port_defs.h"

typedef struct nn_entry {
    struct nn_entry	*next;
    netname_name_t	nne_name;
    port_t		nne_port;
    port_t		nne_signature;
} nn_entry_t, *nn_entry_ptr_t;

#define NN_TABLE_SIZE	32
extern struct lock_queue	nn_table[NN_TABLE_SIZE];

#define NN_NAME_HASH(index, name) { 			\
    register char *cp = (char *)(name);			\
    (index) = 0;					\
    while ((*cp) != '\0') { (index) += *cp; cp++;}	\
    (index) = (index) % NN_TABLE_SIZE;			\
}

#define NN_CONVERT_TO_UPPER(name) {			\
    register char *cp = (char *)(name);			\
    while ((*cp) != '\0') {				\
	if (islower(*cp)) *cp = toupper(*cp);		\
	cp++;						\
    }							\
}


/*
 * Structures used to make and remember about network name requests.
 */
typedef struct {
    disp_hdr_t		nnr_disp_hdr;
    netname_name_t	nnr_name;
    network_port_t	nnr_nport;
} nn_req_t, *nn_req_ptr_t;

typedef struct {
    struct condition	nnrr_condition;
    struct mutex	nnrr_lock;
    port_t		nnrr_lport;
    int			nnrr_result;
} nn_req_rec_t, *nn_req_rec_ptr_t;


/*
 * Functions public to the network name service module.
 */
extern nn_handle_request();
extern nn_handle_reply();
extern nn_name_test();
extern nn_network_look_up();

/*
 * Memory management definitions.
 */
extern mem_objrec_t		MEM_NNREC;


#endif	_NN_DEFS_
