/* 
 * Mach Operating System
 * Copyright (c) 1993,1992,1991,1990,1989,1988,1987 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator   or   Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they made and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
/*
 * HISTORY
 * $Log:	sbuf.c,v $
 * Revision 2.2  93/08/10  13:24:20  mrt
 * 	First checkin to Mach 3.0 RCS tree
 * 
 * Revision 2.2  91/11/13  17:20:07  dpj
 * 	Added debugging version of sbuf_extract().
 * 	[91/11/06  20:52:47  dpj]
 * 
 * Revision 2.1  90/10/27  20:46:06  dpj
 * Moving under MACH3 source control
 * 
 * Revision 1.8  89/05/02  11:17:07  dpj
 * 	Fixed all files to conform to standard copyright/log format
 * 
 * 26-Mar-88  Daniel Julin (dpj) at Carnegie-Mellon University
 *	Converted to use the new memory management module.
 *
 * 28-Nov-86  Robert Sansom (rds) at Carnegie Mellon University
 *	Added sbuf_printf.
 *
 * 25-Nov-86  Robert Sansom (rds) at Carnegie Mellon University
 *	Started.
 *
 */
/*
 * sbuf.c
 *
 *
 */

#ifndef	lint
char sbuf_rcsid[] = "$Header: sbuf.c,v 2.2 93/08/10 13:24:20 mrt Exp $";
#endif not lint

/*
 * Auxiliary functions for sbuf management.
 */


#include <stdio.h>

#include "mem.h"
#include "netmsg.h"
#include "nm_extra.h"
#include "sbuf.h"


/*
 * sbuf_grow
 *	Increase the number of segments in an sbuf.
 *
 * Parameters:
 *	sbuf_ptr	: a pointer to the sbuf to grow
 *	inc		: the number of new segments required
 *
 * Design:
 *	Allocates space for the new segments.
 *	Copies the entries from the old segments into the new ones.
 *	Deallocates the old segments.
 *	Inserts the new segments into the sbuf.
 *
 */
PUBLIC void sbuf_grow(sbuf_ptr, inc)
register sbuf_ptr_t	sbuf_ptr;
int			inc;
BEGIN("sbuf_grow")
    register sbuf_seg_ptr_t	new_segs, old_seg_ptr, new_seg_ptr;
    register int		segs_used;
    int				num_segs;

    DEBUG2(debug.sbuf,1,4000,sbuf_ptr,inc);

    /*
     * Allocate space for the new segments.
     */
    num_segs = sbuf_ptr->size + inc;
    MEM_ALLOC(new_segs,sbuf_seg_ptr_t,(num_segs * sizeof(struct sbuf_seg)), FALSE);

    /*
     * Copy the old segments into the new segments.
     * Also sets new_seg_ptr to point at the first unused segment in new_segs.
     */
    old_seg_ptr = sbuf_ptr->segs;
    new_seg_ptr = new_segs;
    segs_used = 0;
    for (; old_seg_ptr != sbuf_ptr->end; old_seg_ptr++, new_seg_ptr++) {
	*new_seg_ptr = *old_seg_ptr;
	segs_used ++;
    }

    /*
     * Dellocate the old segments.
     */
    MEM_DEALLOC((pointer_t)sbuf_ptr->segs, (sbuf_ptr->size * sizeof(struct sbuf_seg)));

    /*
     * Place the new segments into the sbuf.
     */
    sbuf_ptr->segs = new_segs;
    sbuf_ptr->end = new_seg_ptr;
    sbuf_ptr->free = num_segs - segs_used;
    sbuf_ptr->size = num_segs;

END



/*
 * sbuf_printf
 *	print out an sbuf
 *
 * Parameters:
 *	where	: output file
 *	sb_ptr	: a pointer to the sbuf to print
 *
 */
EXPORT void sbuf_printf(where, sb_ptr)
FILE		*where;
sbuf_ptr_t	sb_ptr;
BEGIN("sbuf_printf")
    sbuf_seg_ptr_t	seg_ptr;

    fprintf(where, "sbuf at %d:\n", (long)sb_ptr);
    fprintf(where, "  end = %d, segs = %d, free = %d & size = %d\n",
		 (long)sb_ptr->end, (long)sb_ptr->segs, sb_ptr->free, sb_ptr->size);
    seg_ptr = sb_ptr->segs;
    for (; seg_ptr != sb_ptr->end; seg_ptr++) {
	fprintf(where, "    segment at %d: data = %d, size = %d.\n",
			(long)seg_ptr, (long)seg_ptr->p, seg_ptr->s);
    }
    RET;

END


#if	NET_DEBUG
sbuf_extract_debug(sb,pos,addr,count)
	sbuf_t		*sb;
	sbuf_pos_t	*pos;
	pointer_t	*addr;
	int		count;
{
	register int		total_left = (count);

	while (total_left > (*pos).data_left) {
		bcopy((char *)(*pos).data_ptr,(char *)(*addr),(*pos).data_left);
		total_left -= (*pos).data_left;
		(*addr) = (pointer_t)(((char *)*addr) + (*pos).data_left);
		(*pos).seg_ptr++;
		if ((*pos).seg_ptr == (*sb).end) {
			if (total_left) {
				sbuf_printf(stderr,sb);
				panic("SBUF_EXTRACT");
			}
		}
		else {
			(*pos).data_ptr = (*pos).seg_ptr->p;
			(*pos).data_left = (*pos).seg_ptr->s;
		}
	}
	if (total_left) {
		bcopy((char *)(*pos).data_ptr,(char *)(*addr),total_left);
		(*addr) = (pointer_t)(((char *)*addr) + total_left);
		(*pos).data_ptr += total_left;
		(*pos).data_left -= total_left;
	}
}
#endif	NET_DEBUG
