/* 
 * Mach Operating System
 * Copyright (c) 1993,1992,1991,1990,1989,1988,1987 Carnegie Mellon University
 * All Rights Reserved.
 * 
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 * 
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Carnegie Mellon requests users of this software to return to
 * 
 *  Software Distribution Coordinator   or   Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 * 
 * any improvements or extensions that they made and grant Carnegie Mellon the
 * rights to redistribute these changes.
 */
/*
 * HISTORY
 * $Log:	vmtp1_defs.h,v $
 * Revision 2.2  93/08/10  13:27:44  mrt
 * 	First checkin to Mach 3.0 RCS tree
 * 
 * Revision 2.1  90/10/27  20:47:05  dpj
 * Moving under MACH3 source control
 * 
 * Revision 1.6  89/05/02  11:19:28  dpj
 * 	Fixed all files to conform to standard copyright/log format
 * 
 * 23-Jun-87  Robert Sansom (rds) at Carnegie Mellon University
 *	Added support for encrypted data.
 *
 *  8-Jun-87  Daniel Julin (dpj) at Carnegie-Mellon University
 *	Created.
 */
/*
 * vmtp1_defs.h
 *
 *
 * $Header: vmtp1_defs.h,v 2.2 93/08/10 13:27:44 mrt Exp $
 *
 */

/*
 * Definitions for the VMTP module.
 */


#ifndef	_VMTP1_DEFS_
#define	_VMTP1_DEFS_

#define KERNEL_FEATURES	1

#include	<sys/types.h>
#include	<sys/socket.h>
#include	<netinet/in.h>
#include	<netinet/vmtp_so.h>
#include	<netinet/vmtp.h>

#include	"crypt.h"
#include	"sbuf.h"


/*
 * Maximum VMTP segment size, preallocated in the transport records.
 */
#define	VMTP_SEGSIZE	16384
#define VMTP_MAX_DATA	(VMTP_SEGSIZE - sizeof(crypt_header_t) - sizeof(long))

typedef struct vmtp_segment {
	crypt_header_t	crypt_header;
	char		data[VMTP_MAX_DATA];
	long		pad;
} vmtp_segment_t, *vmtp_segment_ptr_t;

/*
 * VMTP transport record.
 */
typedef struct vmtp_rec {
	vmtp_segment_t	segment;	/* maximum-size VMTP segment */
	struct vmtp_rec	*next;		/* next in free queue */
	int		socket;		/* associated socket (if any) */
	struct vmtpmcb	mcb;		/* VMTP message control block */
	sbuf_t		sbuf;		/* sbuf for data */
	sbuf_seg_t	sbuf_seg;	/* segment for sbuf */
	int		client_id;	/* higher-level (ipc) ID */
	int		(*callback)();	/* client callback procedure */
} vmtp_rec_t, *vmtp_rec_ptr_t;


#endif	_VMTP1_DEFS_
